@@ -3,7 +3,7 @@
 
 from .simple_json import simple_json
 
-PYPI_URL = 'https://pypi.python.org/pypi/%s/json'
+PYPI_URL = 'https://pypi.org/pypi/%s/json'
 
 def _version_from_json(data):
   return data['info']['version']
